<?php
namespace Teknisa\FIS\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Utilities;


class Ger00200EmpresaService{

    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager){
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function salvarInscricaoEstadual($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $dataSet = $this->criaDataSetInscricaoEstadual($row);
            $this->saveInscricao($dataSet);
            $this->gravaCertificado($row);

            $connection->commit();
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function saveInscricao ($dataSet){
        try {
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    const GET_INSCRICAO = "SELECT * FROM INSCRESTAD
                            WHERE CDEMPRESA = :CDEMPRESA
                            AND CDINSCESTA = :CDINSCESTA
                            AND NRORG = :NRORG";

    public function consultaInscricao($row){
        try {
            $params = array(
                'CDEMPRESA' => $row[0]['CDEMPRESA'],
                'CDINSCESTA' => $row[0]['CDINSCESTA'],
                'NRORG' => $this->environment->getNrOrgTrab()
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_INSCRICAO, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public  function criaDataSetInscricaoEstadual ($row){
        if($row[0]['__is_new']){
            if($row[0]['CDSENHACERTIF']){
                $rows = $this->criptografarSenhaCertificado($row);
                $dataSet = new DataSet('inscrestad_completa', $rows);
            }else{
                $dataSet = new DataSet('inscrestad_completa', $row);
            }
            return $dataSet;
        }else{
            /*$conditions = $this->condicaoInscricao($row);
            $resultadoBanco = $this->realizaConsultaBanco($conditions,'inscrestad_completa');
            $inscricao = $resultadoBanco->getRows();*/

            $resultadoBanco = $this->consultaInscricao($row);
            $inscricao = $resultadoBanco;
            
            if($row[0]['CDSENHACERTIF'] !== $inscricao[0]['CDSENHACERTIF']){
                if($row[0]['CDSENHACERTIF']){
                    $rows = $this->criptografarSenhaCertificado($row);
                    $dataSet = new DataSet('inscrestad_completa', $rows);
                }else{
                    $dataSet = new DataSet('inscrestad_completa', $row);
                }
            }else{
                $dataSet = new DataSet('inscrestad_completa', $row);
            }
            return $dataSet;
        }
    }

    public function criptografarSenhaCertificado($row) {
        try {
            $senhaCriptografada = Utilities::zeedhiEncrypt($row[0]['CDSENHACERTIF'], 'teknisa');
            $row[0]['CDSENHACERTIF'] = $senhaCriptografada;
            return $row;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravaCertificado ($row){
        try {
            $inscricao = $row[0];
            $diretorio = $this->criarDiretorio($inscricao);
            $certificado = $inscricao['CERTIFICADO'];
            if($inscricao['__is_new']){
                if(count($certificado) > 0){
                    $this->apagarArquivosDaPasta($diretorio);
                    $file = $certificado[0]['b64File'];
                    $pos = strpos($file, ',');
                    $file = substr($file,$pos+1);
                    file_put_contents($diretorio.DIRECTORY_SEPARATOR.$inscricao['NMCERTIFICADO'].'.pfx', base64_decode($file));
                }
            }else{
                if(count($certificado) > 0){
                    $this->apagarArquivosDaPasta($diretorio);
                    $file = $certificado[0]['b64File'];
                    $pos = strpos($file, ',');
                    $file = substr($file,$pos+1);
                    file_put_contents($diretorio.DIRECTORY_SEPARATOR.$inscricao['NMCERTIFICADO'].'.pfx', base64_decode($file));
                }else{
                    if( ! $inscricao['NMCERTIFICADO']){
                        $this->apagarArquivosDaPasta($diretorio);
                    }
                }

            }
        }catch (\Exception $e) {
            throw $e;
        }

    }

    public function criarDiretorio ($inscricao){
        try {
            $raiz = realpath(__DIR__ . "/../../../../../../../../"). DIRECTORY_SEPARATOR . "anexos" . DIRECTORY_SEPARATOR . "transmissaoNFe";
            $nrorgDir = $raiz . DIRECTORY_SEPARATOR . $inscricao['NRORG'];
            $ieDir    = $nrorgDir . DIRECTORY_SEPARATOR . $inscricao['CDINSCESTA'];
            $certDir = $ieDir . DIRECTORY_SEPARATOR . "cert";
            
            if (!is_dir($certDir)) {
                if (!mkdir($certDir, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $certDir. Verifique as permissões"
                    );
                }
            }
            return $certDir;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function apagarArquivosDaPasta($dir){
        $dir = $dir.'\\';
        if(is_dir($dir)){
            if($handle = opendir($dir)){
                while(($file = readdir($handle)) !== false){
                    if($file != '.' && $file != '..'){
                        unlink($dir.$file);
                    }
                }
            }
        }
    }

    public function condicaoInscricao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row[0]['CDEMPRESA'],
            "operator" => "=",
            "columnName" => "CDEMPRESA"
          ),
          array(
            "value" => $row[0]['CDINSCESTA'],
            "operator" => "=",
            "columnName" => "CDINSCESTA"
          )
        );
        return $conditions;
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }

}